__author__ = 'tcaruso'

import json

import click
import tabulate

from enum import Enum

from nhlstats.cli.helpers import list_of_dicts_to_csv


class OutputFormat(Enum):
    TEXT = 'text'
    CSV = 'csv'
    JSON = 'json'
    POSTGRES = 'postgres'

    @staticmethod
    def options():
        return list(out.value for out in OutputFormat)

    @classmethod
    def from_click_option(cls, ctx, param, value):
        try:
            return cls(value)
        except:
            raise click.BadParameter(value)

    def echo(self, events: list) -> None:
        if self.value == 'text':
            click.echo(tabulate.tabulate(events, headers='keys'))

        elif self.value == 'csv':
            click.echo(list_of_dicts_to_csv(events))

        elif self.value == 'json':
            click.echo(json.dumps({'plays': events}))

        else:
            raise click.UsageError("Output format {} is not implemented.".format(self.value))



__author__ = 'tcaruso'

import datetime

import requests

from nhlstats import normalize


BASE_URL = "https://statsapi.web.nhl.com"
SCHEDULE_URL = BASE_URL + "/api/v1/schedule"
LIVE_PLAYS_URL = BASE_URL + "/api/v1/game/{game_id}/feed/live"


def list_games(start_date=None, end_date=None):
    if start_date is None:
        start_date = str(datetime.date.today())
    if end_date is None:
        end_date = str(datetime.date.today())

    resp = requests.get(SCHEDULE_URL, params={'startDate': start_date, 'endDate': end_date})

    data = resp.json()

    return [normalize.game_summary(game) for date in data['dates'] for game in date['games']]


def list_plays_raw(game_id):
    resp = requests.get(LIVE_PLAYS_URL.format(game_id=game_id))

    data = resp.json()

    if data.get('message'):
        raise Exception("Invalid GAME_ID.")

    return data['liveData']['plays']['allPlays']


def list_plays(game_id):
    return list(map(normalize.event, list_plays_raw(game_id)))


def list_shots(game_id):
    plays = list_plays(game_id)

    return list(filter(
        lambda p: 'SHOT' in p['event_type'] or p['event_type'] == 'GOAL', plays
    ))



__author__ = 'tcaruso'


def game_summary(gs):
    return {
        'date': gs['gameDate'][:10],
        'game_id': gs['gamePk'],
        'home_team': gs['teams']['home']['team']['name'],
        'home_score': gs['teams']['home']['score'],
        'away_team': gs['teams']['away']['team']['name'],
        'away_score': gs['teams']['away']['score'],
        'season': gs['season'],
        'game_state': gs['status']['detailedState'],
    }


def event(ev):
    e = {
        'datetime': ev['about']['dateTime'],
        'period': ev['about']['period'],
        'period_time': ev['about']['periodTime'],
        'period_time_remaining': ev['about']['periodTimeRemaining'],
        'period_type': ev['about']['periodType'],
        'x': ev.get('coordinates', {}).get('x'),
        'y': ev.get('coordinates', {}).get('y'),
        'event_type': ev['result']['eventTypeId'],
        'event_secondary_type': ev['result'].get('secondaryType'),
        'event_description': ev['result'].get('description'),
        'team_for': ev.get('team', {}).get('triCode'),
    }

    for idx, player in enumerate(ev.get('players', [])):
        e['player_{}'.format(idx + 1)] = player.get('player', {}).get('fullName')
        e['player_{}_type'.format(idx + 1)] = player.get('playerType')
        e['player_{}_id'.format(idx + 1)] = player.get('player', {}).get('id')

    return e


__author__ = 'tcaruso'

import re

import click


def ensure_yyymmdd_date(ctx, param, value):
    match = re.search(r'\d\d\d\d-\d\d-\d\d', value)

    if match is None:
        raise click.BadParameter("Date must be of the form YYYY-MM-DD.")

    return value


def normalize_game_summary(gs):
    return {
        'date': gs['gameDate'][:10],
        'game_id': gs['gamePk'],
        'home_team': gs['teams']['home']['team']['name'],
        'home_score': gs['teams']['home']['score'],
        'away_team': gs['teams']['away']['team']['name'],
        'away_score': gs['teams']['away']['score'],
        'season': gs['season'],
        'game_state': gs['status']['detailedState'],
    }


def normalize_event(event):
    e = {
        'datetime': event['about']['dateTime'],
        'period': event['about']['period'],
        'period_time': event['about']['periodTime'],
        'period_time_remaining': event['about']['periodTimeRemaining'],
        'period_type': event['about']['periodType'],
        'x': event.get('coordinates', {}).get('x'),
        'y': event.get('coordinates', {}).get('y'),
        'event_type': event['result']['eventTypeId'],
        'event_secondary_type': event['result'].get('secondaryType'),
        'event_description': event['result'].get('description'),
        'team_for': event.get('team', {}).get('triCode'),
    }

    for idx, player in enumerate(event.get('players', [])):
        e['player_{}'.format(idx + 1)] = player.get('player', {}).get('fullName')
        e['player_{}_type'.format(idx + 1)] = player.get('playerType')
        e['player_{}_id'.format(idx + 1)] = player.get('player', {}).get('id')

    return e


def list_of_dicts_to_csv(ld):
    return ','.join(ld[0].keys()) + '\n' + \
           '\n'.join(','.join(str(val) if (val or val == 0) else '' for val in row.values()) for row in ld)



import io

__author__ = 'tcaruso'

# !/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

try:
    from pip._internal.req import parse_requirements
    from pip._internal.download import PipSession
except ImportError:
    from pip.req import parse_requirements
    from pip.download import PipSession
except Exception:
    from pip import __version__ as __pip_version__

    msg = """Sorry, could not install due to a pip import error. Please open an issue on the repo 
    with this message and the error so it can be addressed.
    pip version: {}
    python version: {}
    """.format(__pip_version__, '.'.join(sys.version_info))
    raise EnvironmentError(msg)

here = os.path.abspath(os.path.dirname(__file__))

# ------------------------------------------------

# Package meta-data.
# PACKAGE_NAME is the name of the package directory and the import path. If you use my_package then when installed, you
# will import the package like `import my_package`.
PACKAGE_NAME = 'nhlstats'
DESCRIPTION = 'Tools for collecting NHL play-by-play stats.'
URL = 'https://github.com/tomplex/nhlstats'
EMAIL = 'carusot42@gmail.com'
AUTHOR = 'Tom Caruso'
# The minimum Python version required
REQUIRES_PYTHON = (3, 5, 0)
# PYPI_NAME is the name of the package on pypi. You'll use this name to install the package.
PYPI_NAME = '{}'.format(PACKAGE_NAME)

# ------------------------------------------------
# setup() info

about = {
    'name': PACKAGE_NAME,
    'description': DESCRIPTION,
    'author': AUTHOR,
    'author_email': EMAIL,
    'url': URL,
    'packages': find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    'include_package_data': True,
    'entry_points': {
        'console_scripts': ['nhl=nhlstats.cli.core:cli'],
    },
    'classifiers': [
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
}

# ------------------------------------------------
# Check Python version we're installing against. Bail if it's not correct. This will blow up both when we build the
# package and when someone tries to install it.

if sys.version_info < REQUIRES_PYTHON:
    # Raise if we're trying to install on an unsupported Python version
    raise Exception("Package {} requires python >= {}.".format(PYPI_NAME, '.'.join(map(str, REQUIRES_PYTHON))))

REQUIRES_PYTHON = '>=' + '.'.join(map(str, REQUIRES_PYTHON))


# ------------------------------------------------
# Requirements gathering.

requirements = parse_requirements(os.path.join(os.path.dirname(__file__), 'requirements.txt'), session=PipSession())
about['install_requires'] = [str(req.req) for req in requirements]

# ------------------------------------------------
# Versions handling

version = {}
with open(os.path.join(here, PYPI_NAME, '__version__.py')) as f:
    exec(f.read(), version)

about['version'] = version['__version__']


# ------------------------------------------------
# description metadata

try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        about['long_description'] = '\n' + f.read()
        about['long_description_content_type'] = 'text/markdown'

except FileNotFoundError:
    about['long_description'] = DESCRIPTION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status("Installing required build packages...")
        os.system('{0} -m pip install wheel twine'.format(sys.executable))

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to pypi via Twine…')
        os.system('{0} -m twine upload dist/* '.format(sys.executable))

        sys.exit()


setup(
    **about,
    cmdclass={
        'upload': UploadCommand,
    },
)
